@@ -76,6 +76,12 @@ class SimpleSAML_Auth_Default {
 
 		/* Save persistent authentication data. */
 		$persistentAuthState = array();
+
+		if (isset($state['IdP'])) {
+			/* For backwards compatibility. */
+			$persistentAuthState['saml:sp:IdP'] = $state['IdP'];
+		}
+
 		if (isset($state['PersistentAuthData'])) {
 			foreach ($state['PersistentAuthData'] as $key) {
 				if (isset($state[$key])) {
@@ -84,6 +90,13 @@ class SimpleSAML_Auth_Default {
 			}
 		}
 
+		/* Add those that should always be included. */
+		foreach (array('Attributes', 'Expires', 'LogoutState', 'AuthnInstant') as $a) {
+			if (isset($state[$a])) {
+				$persistentAuthState[$a] = $state[$a];
+			}
+		}
+
 		return $persistentAuthState;
 	}
 
@@ -105,20 +118,6 @@ class SimpleSAML_Auth_Default {
 		/* Save session state. */
 		$session = SimpleSAML_Session::getInstance();
 		$session->doLogin($state['SimpleSAML_Auth_Default.id'], self::extractPersistentAuthState($state));
-		$session->setAttributes($state['Attributes']);
-		if(array_key_exists('Expires', $state)) {
-			$session->setSessionDuration($state['Expires'] - time());
-		}
-
-		if (array_key_exists('LogoutState', $state)) {
-			$session->setLogoutState($state['LogoutState']);
-		}
-
-		if (array_key_exists('IdP', $state)) {
-			$session->setIdP($state['IdP']);
-		} else {
-			$session->setIdP(NULL);
-		}
 
 		if (is_string($return)) {
 			/* Redirect... */
@@ -137,20 +136,29 @@ class SimpleSAML_Auth_Default {
 	 * will return if the logout operation does not require a redirect.
 	 *
 	 * @param string $returnURL  The URL we should redirect the user to after logging out.
+	 * @param string|NULL $authority  The authentication source we are logging out from, or NULL to log out of the most recent.
 	 */
-	public static function initLogoutReturn($returnURL) {
+	public static function initLogoutReturn($returnURL, $authority = NULL) {
 		assert('is_string($returnURL)');
+		assert('is_string($authority) || is_null($authority)');
 
 		$session = SimpleSAML_Session::getInstance();
 
-		$state = $session->getLogoutState();
-		$authId = $session->getAuthority();
-		$session->doLogout();
+		if ($authority === NULL) {
+			$authority = $session->getAuthority();
+			if ($authority === NULL) {
+				/* Already logged out - nothing to do here. */
+				return;
+			}
+		}
+
+		$state = $session->getAuthData($authority, 'LogoutState');
+		$session->doLogout($authority);
 
 		$state['SimpleSAML_Auth_Default.ReturnURL'] = $returnURL;
 		$state['LogoutCompletedHandler'] = array(get_class(), 'logoutCompleted');
 
-		$as = SimpleSAML_Auth_Source::getById($authId);
+		$as = SimpleSAML_Auth_Source::getById($authority);
 		if ($as === NULL) {
 			/* The authority wasn't an authentication source... */
 			self::logoutCompleted($state);
@@ -167,11 +175,13 @@ class SimpleSAML_Auth_Default {
 	 * never returns.
 	 *
 	 * @param string $returnURL  The URL we should redirect the user to after logging out.
+	 * @param string|NULL $authority  The authentication source we are logging out from, or NULL to log out of the most recent.
 	 */
-	public static function initLogout($returnURL) {
+	public static function initLogout($returnURL, $authority = NULL) {
 		assert('is_string($returnURL)');
+		assert('is_string($authority) || is_null($authority)');
 
-		self::initLogoutReturn($returnURL);
+		self::initLogoutReturn($returnURL, $authority);
 
 		/* Redirect... */
 		SimpleSAML_Utilities::redirect($returnURL);
@@ -241,26 +251,6 @@ class SimpleSAML_Auth_Default {
 		$session = SimpleSAML_Session::getInstance();
 		$session->doLogin($authId, self::extractPersistentAuthState($state));
 
-		if (array_key_exists('Attributes', $state)) {
-			$session->setAttributes($state['Attributes']);
-		} else {
-			$session->setAttributes(array());
-		}
-
-		if(array_key_exists('Expires', $state)) {
-			$session->setSessionDuration($state['Expires'] - time());
-		}
-
-		if (array_key_exists('LogoutState', $state)) {
-			$session->setLogoutState($state['LogoutState']);
-		}
-
-		if (array_key_exists('IdP', $state)) {
-			$session->setIdP($state['IdP']);
-		} else {
-			$session->setIdP(NULL);
-		}
-
 		SimpleSAML_Utilities::redirect($redirectTo);
 	}
 
