@@ -10,7 +10,7 @@
  *
  * @author Andreas kre Solberg, UNINETT AS. <andreas.solberg@uninett.no>
  * @package simpleSAMLphp
- * @version $Id: Session.php 2322 2010-05-26 07:10:09Z olavmrk $
+ * @version $Id: Session.php 2636 2010-11-16 14:30:23Z olavmrk $
  */
 class SimpleSAML_Session {
 
@@ -21,11 +21,30 @@ class SimpleSAML_Session {
 	const DATA_TIMEOUT_LOGOUT = 'logoutTimeout';
 
 
+	/**
+	 * The list of loaded session objects.
+	 *
+	 * This is an associative array indexed with the session id.
+	 *
+	 * @var array
+	 */
+	private static $sessions = array();
+
+
 	/**
 	 * This variable holds the instance of the session - Singleton approach.
 	 */
 	private static $instance = null;
 	
+
+	/**
+	 * The session ID of this session.
+	 *
+	 * @var string|NULL
+	 */
+	private $sessionId;
+
+
 	/**
 	 * The track id is a new random unique identifier that is generate for each session.
 	 * This is used in the debug logs and error messages to easily track more information
@@ -41,8 +60,6 @@ class SimpleSAML_Session {
 	private $sessionindex = null;
 	private $nameid = null;
 	
-	private $sp_at_idpsessions = array();
-	
 	private $authority = null;
 	
 	// Session duration parameters
@@ -104,36 +121,115 @@ class SimpleSAML_Session {
 	private $associations = array();
 
 
+	/**
+	 * The authentication token.
+	 *
+	 * This token is used to prevent session fixation attacks.
+	 *
+	 * @var string|NULL
+	 */
+	private $authToken;
+
+
+	/**
+	 * Authentication data.
+	 *
+	 * This is an array with authentication data for the various authsources.
+	 *
+	 * @var array|NULL  Associative array of associative arrays.
+	 */
+	private $authData;
+
+
 	/**
 	 * private constructor restricts instantiaton to getInstance()
 	 */
 	private function __construct($transient = FALSE) {
 
-		
-		$configuration = SimpleSAML_Configuration::getInstance();
-		$this->sessionduration = $configuration->getInteger('session.duration', 8*60*60);
-		
+		$this->authData = array();
 
 		if ($transient) {
 			$this->trackid = 'XXXXXXXXXX';
 			return;
 		}
 
-		$this->trackid = SimpleSAML_Utilities::generateTrackID();
+		$sh = SimpleSAML_SessionHandler::getSessionHandler();
+		$this->sessionId = $sh->getCookieSessionId();
+
+		$this->trackid = substr(md5(uniqid(rand(), true)), 0, 10);
 
 		$this->dirty = TRUE;
 		$this->addShutdownFunction();
 	}
 
 
+	/**
+	 * Upgrade this session object to use the $authData property.
+	 *
+	 * TODO: Remove in version 1.8.
+	 */
+	private function upgradeAuthData() {
+		$this->authData = array();
+
+		if ($this->authority === NULL || !$this->authenticated) {
+			return;
+		}
+
+		if ($this->authState !== NULL) {
+			$data = $this->authState;
+		} else {
+			$data = array();
+		}
+
+		if ($this->attributes !== NULL) {
+			$data['Attributes'] = $this->attributes;
+		} else {
+			$data['Attributes'] = array();
+		}
+
+		if ($this->idp !== NULL) {
+			$data['saml:sp:IdP'] = $this->idp;
+		}
+
+		if ($this->sessionindex !== NULL) {
+			$data['saml:sp:SessionIndex'] = $this->sessionindex;
+		}
+
+		if ($this->nameid !== NULL) {
+			$data['saml:sp:NameID'] = $this->nameid;
+		}
+
+		$data['AuthnInstant'] = $this->sessionstarted;
+		$data['Expire'] = $this->sessionstarted + $this->sessionduration;
+		$this->sessionstarted = NULL;
+		$this->sessionduration = NULL;
+
+		if ($this->logoutState !== NULL) {
+			$data['LogoutState'] = $this->logoutState;
+		}
+
+
+		if (!empty($this->logout_handlers)) {
+			$data['LogoutHandlers'] = $this->logout_handlers;
+		}
+
+		$this->authData[$this->authority] = $data;
+	}
+
+
 	/**
 	 * This function is called after this class has been deserialized.
 	 */
 	public function __wakeup() {
 		$this->addShutdownFunction();
+
+		/* TODO: Remove for version 1.8. */
+		if ($this->authData === NULL) {
+			$this->upgradeAuthData();
+		}
 	}
-	
-	
+
+
 	/**
 	 * Retrieves the current session. Will create a new session if there isn't a session.
 	 *
@@ -151,8 +247,14 @@ class SimpleSAML_Session {
 		 * handler.
 		 */
 		try {
-			self::$instance = self::loadSession();
+			self::$instance = self::getSession();
 		} catch (Exception $e) {
+			if ($e instanceof SimpleSAML_Error_Exception) {
+				SimpleSAML_Logger::error('Error loading session:');
+				$e->logError();
+			} else {
+				SimpleSAML_Logger::error('Error loading session: ' . $e->getMessage());
+			}
 			/* For some reason, we were unable to initialize this session. Use a transient session instead. */
 			self::useTransientSession();
 			return self::$instance;
@@ -186,6 +288,17 @@ class SimpleSAML_Session {
 	}
 
 
+	/**
+	 * Retrieve the session ID of this session.
+	 *
+	 * @return string|NULL  The session ID, or NULL if this is a transient session.
+	 */
+	public function getSessionId() {
+
+		return $this->sessionId;
+	}
+
+
 	/**
 	 * Get a unique ID that will be permanent for this session.
 	 * Used for debugging and tracing log files related to a session.
@@ -193,7 +306,8 @@ class SimpleSAML_Session {
 	public function getTrackID() {
 		return $this->trackid;
 	}
-	
+
+
 	/**
 	 * Who authorized this session. could be in example saml2, shib13, login,login-admin etc.
 	 */
@@ -230,7 +344,8 @@ class SimpleSAML_Session {
 
 		return $authnRequest;
 	}
-	
+
+
 	/**
 	 * This method sets a cached assoc array to the authentication request cache storage.
 	 *
@@ -239,99 +354,108 @@ class SimpleSAML_Session {
 	 * @param $cache			The assoc array that will be stored.
 	 */
 	public function setAuthnRequest($protocol, $requestid, array $cache) {
-	
+
 		SimpleSAML_Logger::debug('Library - Session: Set authnrequest ' . $protocol . ' time:' . time() . ' size:' . count($cache) . '  id: '. $requestid );
 
 		$type = 'AuthnRequest-' . $protocol;
 		$this->setData($type, $requestid, $cache);
 	}
-	
-
 
 
+	/**
+	 * Set the IdP we are authenticated against.
+	 *
+	 * @param string|NULL $idp  Our current IdP, or NULL if we aren't authenticated with an IdP.
+	 */
 	public function setIdP($idp) {
-	
+		assert('is_string($idp) || is_null($idp)');
+		assert('isset($this->authData[$this->authority])');
+
 		SimpleSAML_Logger::debug('Library - Session: Set IdP to : ' . $idp);
 		$this->dirty = true;
-		$this->idp = $idp;
-	}
-	public function getIdP() {
-		return $this->idp;
-	}
-	
-
-	public function setSessionIndex($sessionindex) {
-		SimpleSAML_Logger::debug('Library - Session: Set sessionindex: ' . $sessionindex);
-		$this->dirty = true;
-		$this->sessionindex = $sessionindex;
-	}
-	public function getSessionIndex() {
-		if($this->sessionindex === NULL) {
-			$this->sessionindex = SimpleSAML_Utilities::generateID();
-		}
-		return $this->sessionindex;
-	}
-	public function setNameID($nameid) {
-		SimpleSAML_Logger::debug('Library - Session: Set nameID: ');
-		$this->dirty = true;
-		$this->nameid = $nameid;
-	}
-	public function getNameID() {
-		if (array_key_exists('value', $this->nameid)) {
-			/*
-			 * This session was saved by an old version of simpleSAMLphp.
-			 * Convert to the new NameId format.
-			 *
-			 * TODO: Remove this conversion once every session uses the new format.
-			 */
-			$this->nameid['Value'] = $this->nameid['value'];
-			unset($this->nameid['value']);
-
-			$this->dirty = TRUE;
+		if ($idp !== NULL) {
+			$this->authData[$this->authority]['saml:sp:IdP'] = $idp;
+		} else {
+			unset($this->authData[$this->authority]['saml:sp:IdP']);
 		}
 
-		return $this->nameid;
 	}
 
 
 	/**
-	 * Get the NameID of the users session to the specified entity.
+	 * Retrieve the IdP we are currently authenticated against.
 	 *
-	 * Deprecated, remove in version 1.7.
-	 *
-	 * @param string $entityType  The type of the entity (saml20-sp-remote, shib13-sp-remote, ...).
-	 * @param string $entityId  The entity id.
-	 * @return array  The name identifier, or NULL if no name identifier is associated with this session.
+	 * @return string|NULL  Our current IdP, or NULL if we aren't authenticated with an IdP.
 	 */
-	public function getSessionNameId($entityType, $entityId) {
-		assert('is_string($entityType)');
-		assert('is_string($entityId)');
-
-		if(!is_array($this->sessionNameId)) {
+	public function getIdP() {
+		if (!isset($this->authData[$this->authority]['saml:sp:IdP'])) {
 			return NULL;
 		}
+		return $this->authData[$this->authority]['saml:sp:IdP'];
+	}
 
-		if(!array_key_exists($entityType, $this->sessionNameId)) {
+
+	/**
+	 * Set the SessionIndex we received from our IdP.
+	 *
+	 * @param string|NULL $sessionindex  Our SessionIndex.
+	 */
+	public function setSessionIndex($sessionindex) {
+		assert('is_string($sessionindex) || is_null($sessionindex)');
+		assert('isset($this->authData[$this->authority])');
+
+		SimpleSAML_Logger::debug('Library - Session: Set sessionindex: ' . $sessionindex);
+		$this->dirty = true;
+		if ($sessionindex !== NULL) {
+			$this->authData[$this->authority]['saml:sp:SessionIndex'] = $sessionindex;
+		} else {
+			unset($this->authData[$this->authority]['saml:sp:SessionIndex']);
+		}
+	}
+
+
+	/**
+	 * Retrieve our SessionIndex.
+	 *
+	 * @return string|NULL  Our SessionIndex.
+	 */
+	public function getSessionIndex() {
+		if (!isset($this->authData[$this->authority]['saml:sp:SessionIndex'])) {
 			return NULL;
 		}
+		return $this->authData[$this->authority]['saml:sp:SessionIndex'];
+	}
 
-		if(!array_key_exists($entityId, $this->sessionNameId[$entityType])) {
+
+	/**
+	 * Set our current NameID.
+	 *
+	 * @param array|NULL $nameid  The NameID we received from the IdP
+	 */
+	public function setNameID($nameid) {
+		assert('is_array($nameid) || is_null($nameid)');
+		assert('isset($this->authData[$this->authority])');
+
+		SimpleSAML_Logger::debug('Library - Session: Set nameID: ');
+		$this->dirty = true;
+		if ($nameid !== NULL) {
+			$this->authData[$this->authority]['saml:sp:NameID'] = $nameid;
+		} else {
+			unset($this->authData[$this->authority]['saml:sp:NameID']);
+		}
+	}
+
+
+	/**
+	 * Get our NameID.
+	 *
+	 * @return array|NULL The NameID we received from the IdP.
+	 */
+	public function getNameID() {
+		if (!isset($this->authData[$this->authority]['saml:sp:NameID'])) {
 			return NULL;
 		}
-
-		$nameId = $this->sessionNameId[$entityType][$entityId];
-		if (array_key_exists('value', $nameId)) {
-			/*
-			 * This session was saved by an old version of simpleSAMLphp.
-			 * Convert to the new NameId format.
-			 *
-			 * TODO: Remove this conversion once every session should use the new format.
-			 */
-			$nameId['Value'] = $nameId['value'];
-			unset($nameId['value']);
-		}
-
-		return $nameId;
+		return $this->authData[$this->authority]['saml:sp:NameID'];
 	}
 
 
@@ -341,25 +465,40 @@ class SimpleSAML_Session {
 	 * If the user already has logged in, the user will be logged out first.
 	 *
 	 * @param string $authority  The authority the user logged in with.
-	 * @param array|NULL $authState  The persistent auth state for this authority.
+	 * @param array|NULL $data  The authentication data for this authority.
 	 */
-	public function doLogin($authority, array $authState = NULL) {
+	public function doLogin($authority, array $data = NULL) {
 		assert('is_string($authority)');
+		assert('is_array($data) || is_null($data)');
 
 		SimpleSAML_Logger::debug('Session: doLogin("' . $authority . '")');
 
 		$this->dirty = TRUE;
 
-		if($this->authenticated) {
+		if (isset($this->authData[$authority])) {
 			/* We are already logged in. Log the user out first. */
-			$this->doLogout();
+			$this->doLogout($authority);
 		}
 
-		$this->authenticated = TRUE;
-		$this->authority = $authority;
-		$this->authState = $authState;
 
-		$this->sessionstarted = time();
+		if ($data === NULL) {
+			$data = array();
+		}
+
+		if (!isset($data['AuthnInstant'])) {
+			$data['AuthnInstant'] = time();
+		}
+		if (!isset($data['Expire'])) {
+			$globalConfig = SimpleSAML_Configuration::getInstance();
+			$data['Expire'] = time() + $globalConfig->getInteger('session.duration', 8*60*60);
+		}
+
+		$this->authData[$authority] = $data;
+		$this->authority = $authority;
+
+		$this->authToken = SimpleSAML_Utilities::generateID();
+		$sessionHandler = SimpleSAML_SessionHandler::getSessionHandler();
+		$sessionHandler->setCookie('SimpleSAMLAuthToken', $this->authToken);
 	}
 
 
@@ -367,35 +506,57 @@ class SimpleSAML_Session {
 	 * Marks the user as logged out.
 	 *
 	 * This function will call any registered logout handlers before marking the user as logged out.
+	 *
+	 * @param string|NULL $authority  The authentication source we are logging out of.
 	 */
-	public function doLogout() {
+	public function doLogout($authority = NULL) {
 
-		SimpleSAML_Logger::debug('Session: doLogout()');
+		SimpleSAML_Logger::debug('Session: doLogout(' . var_export($authority, TRUE) . ')');
+
+		if ($authority === NULL) {
+			if ($this->authority === NULL) {
+				SimpleSAML_Logger::debug('Session: No current authsource - not logging out.');
+				return;
+			}
+			$authority = $this->authority;
+		}
+
+		if (!isset($this->authData[$authority])) {
+			SimpleSAML_Logger::debug('Session: Already logged out of ' . $authority . '.');
+			return;
+		}
 
 		$this->dirty = TRUE;
 
-		$this->callLogoutHandlers();
-
-		$this->authenticated = FALSE;
-		$this->authority = NULL;
-		$this->attributes = NULL;
-		$this->logoutState = NULL;
-		$this->authState = NULL;
-		$this->idp = NULL;
+		$this->callLogoutHandlers($authority);
+		unset($this->authData[$authority]);
+		if ($this->authority === $authority) {
+			$this->authority = NULL;
+		}
 
 		/* Delete data which expires on logout. */
 		$this->expireDataLogout();
 	}
 
 
+	/**
+	 * Set the lifetime of our current authentication session.
+	 *
+	 * @param int $duration  The number of seconds this authentication session is valid.
+	 */
 	public function setSessionDuration($duration) {
+		assert('is_int($duration)');
+		assert('isset($this->authData[$this->authority])');
+
 		SimpleSAML_Logger::debug('Library - Session: Set session duration ' . $duration);
 		$this->dirty = true;
 		$this->sessionduration = $duration;
+
+		$this->authData[$this->authority]['Expire'] = time() + $duration;
 	}
-	
-	
-	/*
+
+
+	/**
 	 * Is the session representing an authenticated user, and is the session still alive.
 	 * This function will return false after the user has timed out.
 	 *
@@ -405,33 +566,45 @@ class SimpleSAML_Session {
 	public function isValid($authority) {
 		assert('is_string($authority)');
 
-		SimpleSAML_Logger::debug('Library - Session: Check if session is valid.' .
-			' checkauthority:' . $authority .
-			' thisauthority:' . (isset($this->authority) ? $this->authority : 'null') .
-			' isauthenticated:' . ($this->isAuthenticated() ? 'yes' : 'no') . 
-			' remainingtime:' . $this->remainingTime());
-			
-		if (!$this->isAuthenticated()) return false;
-
-		if ($authority !== $this->authority) {
+		if (!isset($this->authData[$authority])) {
+			SimpleSAML_Logger::debug('Session: '. var_export($authority, TRUE) .' not valid because we are not authenticated.');
 			return FALSE;
 		}
 
-		return $this->remainingTime() > 0;
-	}
-	
-	/*
-	 * If the user is authenticated, how much time is left of the session.
-	 */
-	public function remainingTime() {
-		return $this->sessionduration - (time() - $this->sessionstarted);
+		if ($this->authData[$authority]['Expire'] <= time()) {
+			SimpleSAML_Logger::debug('Session: ' . var_export($authority, TRUE) .' not valid because it is expired.');
+			return FALSE;
+		}
+
+		SimpleSAML_Logger::debug('Session: Valid session found with ' . var_export($authority, TRUE) . '.');
+
+		return TRUE;
 	}
 
-	/* 
+
+	/**
+	 * If the user is authenticated, how much time is left of the session.
+	 *
+	 * @return int  The number of seconds until the session expires.
+	 */
+	public function remainingTime() {
+
+		if (!isset($this->authData[$this->authority])) {
+			/* Not authenticated. */
+			return -1;
+		}
+
+		assert('isset($this->authData[$this->authority]["Expire"])');
+		return $this->authData[$this->authority]['Expire'] - time();
+	}
+
+	/**
 	 * Is the user authenticated. This function does not check the session duration.
+	 *
+	 * @return bool  TRUE if the user is authenticated, FALSE otherwise.
 	 */
 	public function isAuthenticated() {
-		return $this->authenticated;
+		return isset($this->authData[$this->authority]);
 	}
 
 
@@ -441,57 +614,71 @@ class SimpleSAML_Session {
 	 * @return int|NULL  The timestamp for when the user was authenticated. NULL if the user hasn't authenticated.
 	 */
 	public function getAuthnInstant() {
-		if (!$this->isAuthenticated()) {
+
+		if (!isset($this->authData[$this->authority])) {
+			/* Not authenticated. */
 			return NULL;
 		}
 
-		return $this->sessionstarted;
-	}
-	
-	
-	// *** Attributes ***
-	
-	public function getAttributes() {
-		return $this->attributes;
+		assert('isset($this->authData[$this->authority]["AuthnInstant"])');
+		return $this->authData[$this->authority]['AuthnInstant'];
 	}
 
-	public function getAttribute($name) {
-		return $this->attributes[$name];
-	}
 
-	public function setAttributes($attributes) {
-		$this->dirty = true;
-		$this->attributes = $attributes;
-	}
-	
-	public function setAttribute($name, $value) {
-		$this->dirty = true;
-		$this->attributes[$name] = $value;
-	}
-	
 	/**
-	 * Clean the session object.
+	 * Retrieve the attributes associated with this session.
+	 *
+	 * @return array|NULL  The attributes.
 	 */
-	public function clean($cleancache = false) {
-	
-		SimpleSAML_Logger::debug('Library - Session: Cleaning Session. Clean cache: ' . ($cleancache ? 'yes' : 'no') );
-	
-		if ($cleancache) {
-			$this->dataStore = null;
-			$this->idp = null;
+	public function getAttributes() {
+		if (!isset($this->authData[$this->authority]['Attributes'])) {
+			return NULL;
 		}
-		
-		$this->authority = null;
-	
-		$this->authenticated = null;
-		$this->attributes = null;
-	
-		$this->sessionindex = null;
-		$this->nameid = null;
-	
-		$this->dirty = TRUE;
+		return $this->authData[$this->authority]['Attributes'];
 	}
-	 
+
+
+	/**
+	 * Retrieve a single attribute.
+	 *
+	 * @param string $name  The name of the attribute.
+	 * @return array|NULL  The values of the given attribute.
+	 */
+	public function getAttribute($name) {
+		if (!isset($this->authData[$this->authority]['Attributes'][$name])) {
+			return NULL;
+		}
+		return $this->authData[$this->authority]['Attributes'][$name];
+	}
+
+
+	/**
+	 * Set the attributes for this session.
+	 *
+	 * @param array|NULL $attributes  The attributes of this session.
+	 */
+	public function setAttributes($attributes) {
+		assert('isset($this->authData[$this->authority])');
+
+		$this->dirty = true;
+		$this->authData[$this->authority]['Attributes'] = $attributes;
+	}
+
+
+	/**
+	 * Set the values of a single attribute.
+	 *
+	 * @param string $name  The name of the attribute.
+	 * @param array $value  The values of the attribute.
+	 */
+	public function setAttribute($name, $value) {
+		assert('isset($this->authData[$this->authority])');
+
+		$this->dirty = true;
+		$this->authData[$this->authority]['Attributes'][$name] = $value;
+	}
+
+
 	/**
 	 * Calculates the size of the session object after serialization
 	 *
@@ -510,6 +697,7 @@ class SimpleSAML_Session {
 	 * @param $functionname  The logout handler function.
 	 */
 	public function registerLogoutHandler($classname, $functionname) {
+		assert('isset($this->authData[$this->authority])');
 
 		$logout_handler = array($classname, $functionname);
 
@@ -519,16 +707,24 @@ class SimpleSAML_Session {
 		}
 
 
-		$this->logout_handlers[] = $logout_handler;
+		$this->authData[$this->authority]['LogoutHandlers'][] = $logout_handler;
 		$this->dirty = TRUE;
 	}
 
 
 	/**
 	 * This function calls all registered logout handlers.
+	 *
+	 * @param string $authority  The authentication source we are logging out from.
 	 */
-	private function callLogoutHandlers() {
-		foreach($this->logout_handlers as $handler) {
+	private function callLogoutHandlers($authority) {
+		assert('is_string($authority)');
+		assert('isset($this->authData[$authority])');
+
+		if (empty($this->authData[$authority]['LogoutHandlers'])) {
+			return;
+		}
+		foreach($this->authData[$authority]['LogoutHandlers'] as $handler) {
 
 			/* Verify that the logout handler is a valid function. */
 			if(!is_callable($handler)) {
@@ -545,7 +741,7 @@ class SimpleSAML_Session {
 		}
 
 		/* We require the logout handlers to register themselves again if they want to be called later. */
-		$this->logout_handlers = array();
+		unset($this->authData[$authority]['LogoutHandlers']);
 	}
 
 
@@ -764,28 +960,52 @@ class SimpleSAML_Session {
 	/**
 	 * Load a session from the session handler.
 	 *
+	 * @param string|NULL $sessionId  The session we should load, or NULL to load the current session.
 	 * @return The session which is stored in the session handler, or NULL if the session wasn't found.
 	 */
-	private static function loadSession() {
+	public static function getSession($sessionId = NULL) {
+		assert('is_string($sessionId) || is_null($sessionId)');
 
 		$sh = SimpleSAML_SessionHandler::getSessionHandler();
-		$sessionData = $sh->get('SimpleSAMLphp_SESSION');
-		if($sessionData == NULL) {
+
+		if ($sessionId === NULL) {
+			$checkToken = TRUE;
+			$sessionId = $sh->getCookieSessionId();
+		} else {
+			$checkToken = FALSE;
+		}
+
+		if (isset(self::$sessions[$sessionId])) {
+			return self::$sessions[$sessionId];
+		}
+
+
+		$session = $sh->loadSession($sessionId);
+		if($session === NULL) {
 			return NULL;
 		}
 
-		if(!is_string($sessionData)) {
-			return NULL;
+		assert('$session instanceof self');
+
+		/* For backwardscompatibility. Remove after 1.7. */
+		if ($session->sessionId === NULL) {
+			$session->sessionId = $sh->getCookieSessionId();
 		}
 
-		$sessionData = unserialize($sessionData);
-
-		if(!($sessionData instanceof self)) {
-			SimpleSAML_Logger::warning('Retrieved and deserialized session data was not a session.');
-			return NULL;
+		if ($checkToken && $session->authToken !== NULL) {
+			if (!isset($_COOKIE['SimpleSAMLAuthToken'])) {
+				SimpleSAML_Logger::warning('Missing AuthToken cookie.');
+				return NULL;
+			}
+			if ($_COOKIE['SimpleSAMLAuthToken'] !== $session->authToken) {
+				SimpleSAML_Logger::warning('Invalid AuthToken cookie.');
+				return NULL;
+			}
 		}
 
-		return $sessionData;
+		self::$sessions[$sessionId] = $session;
+
+		return $session;
 	}
 
 
@@ -802,10 +1022,18 @@ class SimpleSAML_Session {
 		}
 
 		$this->dirty = FALSE;
-		$sessionData = serialize($this);
 
 		$sh = SimpleSAML_SessionHandler::getSessionHandler();
-		$sh->set('SimpleSAMLphp_SESSION', $sessionData);
+
+		try {
+			$sh->saveSession($this);
+		} catch (Exception $e) {
+			if (!($e instanceof SimpleSAML_Error_Exception)) {
+				$e = new SimpleSAML_Error_UnserializableException($e);
+			}
+			SimpleSAML_Logger::error('Unable to save session.');
+			$e->logError();
+		}
 	}
 
 
@@ -822,11 +1050,11 @@ class SimpleSAML_Session {
 	 *
 	 * @param array $state  The state array.
 	 */
-	public function setLogoutState($state) {
-		assert('is_array($state)');
+	public function setLogoutState(array $state) {
+		assert('isset($this->authData[$this->authority])');
 
 		$this->dirty = TRUE;
-		$this->logoutState = $state;
+		$this->authData[$this->authority]['LogoutState'] = $state;
 	}
 
 
@@ -836,31 +1064,34 @@ class SimpleSAML_Session {
 	 * @return array  The logout state. If no logout state is set, an empty array will be returned.
 	 */
 	public function getLogoutState() {
+		assert('isset($this->authData[$this->authority])');
 
-		if ($this->logoutState === NULL) {
+		if (!isset($this->authData[$this->authority]['LogoutState'])) {
 			return array();
 		}
 
-		return $this->logoutState;
+		return $this->authData[$this->authority]['LogoutState'];
 	}
 
 
 	/**
 	 * Get the current persistent authentication state.
 	 *
+	 * @param string|NULL $authority  The authority to retrieve the data from.
 	 * @return array  The current persistent authentication state, or NULL if not authenticated.
 	 */
-	public function getAuthState() {
-		if (!$this->isAuthenticated()) {
+	public function getAuthState($authority = NULL) {
+		assert('is_string($authority) || is_null($authority)');
+
+		if ($authority === NULL) {
+			$authority = $this->authority;
+		}
+
+		if (!isset($this->authData[$authority])) {
 			return NULL;
 		}
 
-		if (!isset($this->authState)) {
-			/* No AuthState for this login handler. */
-			return array();
-		}
-
-		return $this->authState;
+		return $this->authData[$authority];
 	}
 
 
@@ -878,48 +1109,6 @@ class SimpleSAML_Session {
 	}
 
 
-	/**
-	 * Upgrade the association list to the new format.
-	 *
-	 * Should be removed in version 1.7.
-	 *
-	 * @param string $idp  The IdP we should add the associations to.
-	 */
-	private function upgradeAssociations($idp) {
-		assert('is_string($idp)');
-
-		$sp_at_idpsessions = $this->sp_at_idpsessions;
-		$this->sp_at_idpsessions = NULL;
-		$this->dirty = TRUE;
-
-		$globalConfig = SimpleSAML_Configuration::getInstance();
-		$sessionLifetime = time() + $globalConfig->getInteger('session.duration', 8*60*60);
-
-		foreach ($sp_at_idpsessions as $spEntityId => $state) {
-
-			if ($state !== 1) { /* 1 == STATE_ONLINE */
-				continue;
-			}
-
-			$nameId = $this->getSessionNameId('saml20-sp-remote', $spEntityId);
-			if($nameId === NULL) {
-				$nameId = $this->getNameID();
-			}
-
-			$id = 'saml:' . $spEntityId;
-
-			$this->addAssociation($idp, array(
-				'id' => $id,
-				'Handler' => 'sspmod_saml_IdP_SAML2',
-				'Expires' => $sessionLifetime,
-				'saml:entityID' => $spEntityId,
-				'saml:NameID' => $nameId,
-				'saml:SessionIndex' => $this->getSessionIndex(),
-			));
-		}
-	}
-
-
 	/**
 	 * Add an SP association for an IdP.
 	 *
@@ -933,11 +1122,6 @@ class SimpleSAML_Session {
 		assert('isset($association["id"])');
 		assert('isset($association["Handler"])');
 
-		if (substr($idp, 0, 6) === 'saml2:' && !empty($this->sp_at_idpsessions)) {
-			/* Remove in 1.7. */
-			$this->upgradeAssociations($idp);
-		}
-
 		if (!isset($this->associations)) {
 			$this->associations = array();
 		}
@@ -1052,6 +1236,22 @@ class SimpleSAML_Session {
 		return $ret;
 	}
 
-}
 
-?>
\ No newline at end of file
+	/**
+	 * Retrieve authentication data.
+	 *
+	 * @param string $authority  The authentication source we should retrieve data from.
+	 * @param string $name  The name of the data we should retrieve.
+	 * @return mixed  The value, or NULL if the value wasn't found.
+	 */
+	public function getAuthData($authority, $name) {
+		assert('is_string($authority)');
+		assert('is_string($name)');
+
+		if (!isset($this->authData[$authority][$name])) {
+			return NULL;
+		}
+		return $this->authData[$authority][$name];
+	}
+
+}
