@@ -5,7 +5,7 @@
  *
  * @author Andreas Åkre Solberg, UNINETT AS. <andreas.solberg@uninett.no>
  * @package simpleSAMLphp
- * @version $Id: Utilities.php 2856 2011-05-27 10:39:52Z olavmrk $
+ * @version $Id: Utilities.php 3010 2012-01-10 13:12:38Z olavmrk $
  */
 class SimpleSAML_Utilities {
 
@@ -1081,6 +1081,29 @@ class SimpleSAML_Utilities {
 	}
 
 
+	/**
+	 * Normalizes an URL to an absolute URL and validate it.
+	 *
+	 * In addition to resolving the URL, this function makes sure that it is
+	 * a link to a http or https site.
+	 *
+	 * @param string $url  The relative URL.
+	 * @return string  An absolute URL for the given relative URL.
+	 */
+	public static function normalizeURL($url) {
+		assert('is_string($url)');
+
+		$url = SimpleSAML_Utilities::resolveURL($url, SimpleSAML_Utilities::selfURL());
+
+		/* Verify that the URL is to a http or https site. */
+		if (!preg_match('@^https?://@i', $url)) {
+			throw new SimpleSAML_Error_Exception('Invalid URL: ' . $url);
+		}
+
+		return $url;
+	}
+
+
 	/**
 	 * Parse a query string into an array.
 	 *
